#ifndef _TS_Buffer_H
#define _TS_Buffer_H

#include "Prefix.h"
#include "EARTH_PT.h"
#include "TS_Utility.h"

namespace EARTH {
namespace TS {
	class Buffer {
	public:
		Buffer(PT::Device *device) : m_cDevice(device)
		{
			for(int n = 0; n < (sizeof(m_hStream) / sizeof(m_hStream[0])); ++n){
				m_hStream[n] = INVALID_HANDLE_VALUE;
			}
			m_uTsSize   = 8 * 1024 * 1024;
			m_pTsBuffer = new byte[m_uTsSize];
			m_hThread   = NULL;
			m_bQuit     = false;
		}

		~Buffer()
		{
			Stop();
			if(m_pTsBuffer != NULL){
				delete	[] m_pTsBuffer;
				m_pTsBuffer = NULL;
			}
		}

		void Run()
		{
			status	status;
			TCHAR	dir[MAX_PATH];

			GetCurrentDirectory(MAX_PATH,dir);
			for(int n = 0; n < (sizeof(m_hStream) / sizeof(m_hStream[0])); ++n){
				int  tuner = n / 2;
				int  isdb  = n % 2;
				bool boolEnable;
				m_cDevice->GetStreamEnable(tuner,static_cast<PT::Device::ISDB>(isdb),&boolEnable);
				if(boolEnable){
					TCHAR	file[MAX_PATH];
					wsprintf(file,L"%s\\Tuner%d ISDB-%c.ts",dir,n / 2,(n & 1)? L'T': L'S');
					m_hStream[n] = CreateFile(file,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
				}
			}

			status = m_cDevice->ResetTransferCounter();
			if(Error(L"Device::ResetTransferCounter()", status)) return;

			status = m_cDevice->SetTransferEnable(true);
			if(Error(L"Device::SetTransferEnable()",status)) return;

			m_bQuit = false;
			m_hThread = CreateThread(NULL,0,Thread_,this,0,NULL);
		}

		void Stop()
		{
			status	status;

			bool	transferEnabled;
			status = m_cDevice->GetTransferEnable(&transferEnabled);
			if(Error(L"Device::GetTransferEnable()", status)) return;

			if (transferEnabled) {
				status = m_cDevice->SetTransferEnable(false);
				Error(L"Device::SetTransferEnable()", status);
			}

			if(m_hThread){
				if(m_bQuit == false){
					m_bQuit = true;
					if(WaitForSingleObjectEx(m_hThread,100,FALSE) != WAIT_OBJECT_0)	TerminateThread(m_hThread,0);
				}
				m_hThread = NULL;
			}

			for(int n = 0; n < (sizeof(m_hStream) / sizeof(m_hStream[0])); ++n){
				if(m_hStream[n] == INVALID_HANDLE_VALUE)	continue;
				CloseHandle(m_hStream[n]);
				m_hStream[n] = INVALID_HANDLE_VALUE;
			}
		}

	protected:
		PT::Device *const	m_cDevice;
		HANDLE				m_hStream[2*2];
		HANDLE				m_hThread;
		byte				*m_pTsBuffer;
		uint				m_uTsSize;
		bool				m_bQuit;

		bool DmaErrorCheck()
		{
			bool	err = false;;
			PT::Device::TransferInfo	info;

			status status = m_cDevice->GetTransferInfo(&info);
			if(Error(L"Device::GetTransferInfo()",status))	return false;
			if(info.TransferCounter0){
				wprintf(L"]JE^ 0 ȉɂȂ܂B\n");
				err = true;
			}else if (info.BufferOverflow) {
				wprintf(L"PCI oX𒷊ɓnmۂłȂ߁A{[h FIFO ܂B\n");
				err = true;
			}
			if(err){
				::wprintf(L"t@Cւ̏o𒆎~܂B\n");
				return false;
			}
			return true;
		}

		static uint __stdcall Thread_(void *argument)
		{
			Buffer *that = static_cast<Buffer *>(argument);
			ExitThread(that->Thread());
		}

		uint Thread()
		{
			uint	tsSize;
			__int64	total;
			ULONG	nBytesWrite;
			status	status;

			for(total = 0; m_bQuit == false; Sleep(10)){
				for(int n = 0; n < (sizeof(m_hStream) / sizeof(m_hStream[0])); ++n){
					if(DmaErrorCheck() == false){
						m_bQuit = true;
						break;
					}
					if(m_hStream[n] == INVALID_HANDLE_VALUE)	continue;
					status = m_cDevice->GetStreamData(n / 2,(PT::Device::ISDB )(n & 1),m_pTsBuffer,m_uTsSize,&tsSize);
					if(status != PT::STATUS_OK)	continue;
					if(tsSize == 0)	continue;
					BOOL rc = WriteFile(m_hStream[n],m_pTsBuffer,tsSize,&nBytesWrite,NULL);
					if((rc == FALSE) || (nBytesWrite != tsSize)){
						status = HRESULT_FROM_WIN32(GetLastError());
						if(Error(L"WriteFile()",status)) continue;
					}
					total += tsSize;
				}
			}
			return 0;
		}
	};
}
}

#endif

