//------------------------------------------------------------------------------
// File: CPtDriver.cpp
//
// Date: 2011.05.02
//
// Written by Mr. Wakai.
//------------------------------------------------------------------------------
#include <windows.h>
#include <windowsx.h>
#include <string.h>

#include "PtDrvIfLib.h"
#include "PtDriver.h"
#include "resource.h"


// ----------function prototypes----------
static LRESULT CALLBACK PtDlgProc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam);

static HRESULT PtDrvSetup(void);
static void InitDiaLog(void);
static void delComboItem(HWND hItem);
static void ChannelScan(void);
static int GetOutputFileName();

// ------------data----------------
static	HINSTANCE	m_hInst;
static	HWND		m_hWndApp = NULL;
static	PVOID		m_hDevNotify = NULL;
static	CPtDriver	*m_pPtDriver = NULL; 

static	int	m_iTunerDevice = -1;
static	int	m_iTunerRemocon = -1;
static	int	m_iPtDevCount = 0;
static	int	m_iTunerCount = 0;
static	wchar_t	m_wcTunerName[10*2][16];
static	int	m_iPtScanCount = 0;
static	int	m_iPtScanLink[32];
static	struct	{
	ptScanData	scan;
	int			channel;
	wchar_t		chName[16];
}m_sPtScanData[32];

static	bool	m_fRun = false;
static	int		m_iChannel = 0;
static	wchar_t	m_wcRecFileName[MAX_PATH];

/*-------------------------------------------------------------------------
|   Routine:  WinMain
  ------------------------------------------------------------------------*/
int WINAPI WinMain( HINSTANCE hinst,HINSTANCE hinstPrev,LPSTR pszCmd,int nCmdShow)
{
	ASSERT(!hinstPrev);

	m_hInst = hinst;
	DialogBox(m_hInst,MAKEINTRESOURCE(IDD_PTDIALOG),NULL,(DLGPROC )PtDlgProc);
	return 0;
}

/*-------------------------------------------------------------------------
|   Routine:  PtDlgProc
  ------------------------------------------------------------------------*/ 
LRESULT CALLBACK PtDlgProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{

	switch(msg){
	case WM_INITDIALOG:
	{
		m_hWndApp = hwnd;
		return TRUE;
	}
	case WM_SHOWWINDOW:
	{
		if(PtDrvSetup() == S_OK){
			InitDiaLog();
		}
		return TRUE;
	}
	case WM_COMMAND:
	{
		int	event = HIWORD(wParam);
		int	item  = LOWORD(wParam);
		HWND hItem = GetDlgItem(hwnd,item);
		if(item == IDOK){
			EndDialog(hwnd,IDOK);
			return TRUE;
		}
		if(item == IDCANCEL){
			EndDialog(hwnd,IDCANCEL);
			return TRUE;
		}

		int		index;
		if((item == IDC_COMBO_DEV_SELECT) && (event == CBN_SELCHANGE)){
			index = SendMessage(hItem,CB_GETCURSEL,0,0);
			if(m_iTunerDevice == index)	break;
			if(m_pPtDriver->set_devName(m_wcTunerName[index]) != S_OK){
				MessageBox(NULL,TEXT("w`[i[͎gpł܂!"),TEXT("Error"),0);
				SendMessage(hItem,CB_SETCURSEL,m_iTunerDevice,0);
				m_pPtDriver->set_devName(m_wcTunerName[m_iTunerDevice]);
				return TRUE;
			}
			m_iTunerDevice = index;
			return TRUE;
		}
		else if((item == IDC_COMBO_CH_SELECT) && (event == CBN_SELCHANGE)){
			index = SendMessage(hItem,CB_GETCURSEL,0,0);
			if(m_iTunerRemocon == index)	break;
			m_iTunerRemocon = index;
			int remocon = m_iPtScanLink[index];
			m_iChannel = m_sPtScanData[remocon].channel;
			return TRUE;
		}
		else if((item == IDC_BUTTON_TUNER_SCAN) && (event == BN_CLICKED)){
			EnableWindow(GetDlgItem(m_hWndApp,IDC_COMBO_DEV_SELECT),FALSE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_COMBO_CH_SELECT),FALSE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_TUNER_SCAN),FALSE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_EDITSAVE_FILE_NAME),FALSE);
//			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_SAVE_FILE_PATH),FALSE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_REC_EXECUTE),FALSE);
			ChannelScan();
			EnableWindow(GetDlgItem(m_hWndApp,IDC_COMBO_DEV_SELECT),TRUE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_COMBO_CH_SELECT),TRUE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_TUNER_SCAN),TRUE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_EDITSAVE_FILE_NAME),TRUE);
//			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_SAVE_FILE_PATH),TRUE);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_REC_EXECUTE),TRUE);
			return TRUE;
		}
		else if((item == IDC_BUTTON_SAVE_FILE_PATH) && (event == BN_CLICKED)){
			if(GetOutputFileName() == 0){
				Edit_SetText(GetDlgItem(m_hWndApp,IDC_EDITSAVE_FILE_NAME),m_wcRecFileName);
			}
			return TRUE;
		}
		else if((item == IDC_BUTTON_REC_EXECUTE) && (event == BN_CLICKED)){
			if(m_fRun){
				m_pPtDriver->exe_stopRec();
				Edit_SetText(GetDlgItem(m_hWndApp,IDC_BUTTON_REC_EXECUTE),L"ۑ");
				m_fRun = false;
			}else{
				Edit_GetText(GetDlgItem(m_hWndApp,IDC_EDITSAVE_FILE_NAME),m_wcRecFileName,STR_MAX_LENGTH);
				if(m_pPtDriver->exe_startRec(m_iChannel,m_wcRecFileName) != 0)	return TRUE;
				Edit_SetText(GetDlgItem(m_hWndApp,IDC_BUTTON_REC_EXECUTE),L"~");
				m_fRun = true;
			}
			EnableWindow(GetDlgItem(m_hWndApp,IDC_COMBO_DEV_SELECT),!m_fRun);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_COMBO_CH_SELECT),!m_fRun);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_TUNER_SCAN),!m_fRun);
//			EnableWindow(GetDlgItem(m_hWndApp,IDC_BUTTON_SAVE_FILE_PATH),!m_fRun);
			EnableWindow(GetDlgItem(m_hWndApp,IDC_EDITSAVE_FILE_NAME),!m_fRun);
			return TRUE;
		}
		break;
	}
	default:	return DefWindowProc(hwnd,msg,wParam,lParam);
	}
	return 0;
}

/*-------------------------------------------------------------------------
|   Routine:        GetOutputFileName
  ------------------------------------------------------------------------*/
int GetOutputFileName()
{
	OPENFILENAME	ofn;
	wchar_t	szFileName[_MAX_PATH];

	szFileName[0] = L'\0';
	ZeroMemory(&ofn,sizeof(ofn));
	ofn.lStructSize     = sizeof(OPENFILENAME);
	ofn.hwndOwner       = NULL;
	ofn.lpstrFilter     = L"TS ̧ (*.ts)\0*.ts\0All Files (*.*)\0*.*\0\0";
	ofn.nFilterIndex    = 0;
	ofn.lpstrFile       = szFileName;
	ofn.nMaxFile        = NUMELMS(szFileName);
	ofn.lpstrFileTitle  = NULL;
	ofn.lpstrTitle      = L"݃t@CI";
	ofn.nMaxFileTitle   = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
	if(GetSaveFileName(&ofn) == 0)	-1;
	StringCchCopy(m_wcRecFileName,NUMELMS(m_wcRecFileName),szFileName);

	return 0;
}
/*-------------------------------------------------------------------------
|   Routine: PtDrvSetup
  ------------------------------------------------------------------------*/
HRESULT PtDrvSetup(void)
{   
	HRESULT	hr = S_OK;

	m_pPtDriver = new CPtDriver;
	if(!m_pPtDriver)	return E_OUTOFMEMORY;

	return hr;
}

/*-------------------------------------------------------------------------
|   Routine: InitDiaLog
  ------------------------------------------------------------------------*/
void InitDiaLog(void)
{
	int		index;
	HWND	hItem;

	if(m_pPtDriver == NULL)	return;
	m_iPtDevCount = PtDrvGetDevCount();
	hItem = GetDlgItem(m_hWndApp,IDC_COMBO_DEV_SELECT);
	index = 0;
	for(int dev = 0; dev++ < m_iPtDevCount; ){
		StringCchPrintf(m_wcTunerName[index],NUMELMS(m_wcTunerName[index]),PT_DEV_BT_NAME,dev,1); ++index;
		StringCchPrintf(m_wcTunerName[index],NUMELMS(m_wcTunerName[index]),PT_DEV_BT_NAME,dev,2); ++index;
	}
	m_iTunerCount = index;
	for(index = 0; index < m_iTunerCount; ++index){
		SendMessage(hItem,CB_ADDSTRING,index,(LPARAM )m_wcTunerName[index]);
	}
	for(index = 0; index < m_iTunerCount; ++index){
		if(m_pPtDriver->set_devName(m_wcTunerName[index]) != S_OK)	continue;
		SendMessage(hItem,CB_SETCURSEL,index,0);
		m_iTunerDevice = index;
		break;
	}
}

/*-------------------------------------------------------------------------
|   Routine: delComboItem
  ------------------------------------------------------------------------*/
void delComboItem(HWND hItem)
{
	while(SendMessage(hItem,CB_GETCOUNT,0,0) != 0) SendMessage(hItem,CB_DELETESTRING,0,0);
}


/*-------------------------------------------------------------------------
|   Routine: ChannelScan
  ------------------------------------------------------------------------*/
void ChannelScan()
{
	int		i;
	int		min;
	int		chMax;
	int		index;
	HWND	hItem;
	int		channel;
	int		frequency;
	wchar_t const	*pChName;
	bool	fMark[NUMELMS(m_sPtScanData)];
	wchar_t	sz[STR_MAX_LENGTH];

	if(m_pPtDriver == NULL)	return;
	m_iPtScanCount = 0;
	hItem = GetDlgItem(m_hWndApp,IDC_COMBO_CH_SELECT);
	delComboItem(hItem);

	m_pPtDriver->get_chMax(&chMax);
	for(index = 0,channel = 0; channel < chMax; ++channel){
		if(m_pPtDriver->get_chInfo(channel,&frequency,&pChName) != S_OK)	return;
		if(pChName[0] == L'C')	continue;
		if(m_pPtDriver->exe_startScan(channel,&m_sPtScanData[index].scan) != S_OK)	continue;
		m_sPtScanData[index].channel = channel;
		StringCchCopy(m_sPtScanData[index].chName,NUMELMS(m_sPtScanData[index].chName),pChName);
		if(++index > NUMELMS(m_sPtScanData))	break;
	}
	memset(fMark,0,sizeof(fMark));
	m_iPtScanCount = index;
	int levelLow = 0;
	for(index = 0; index < m_iPtScanCount; ++index){
		for(min = 0; min < m_iPtScanCount; ++min)	if(fMark[min] == false)	break;
		if(min >= m_iPtScanCount)	break;
		if(m_sPtScanData[min].scan.level < 2600){
			fMark[min] = true;
			++levelLow;
			continue;
		}
		int remocon = m_sPtScanData[min].scan.remocon;
		for(i = min + 1; i < m_iPtScanCount; ++i){
			if(fMark[i] == true)	continue;
			if(remocon <= m_sPtScanData[i].scan.remocon)	continue;
			remocon = m_sPtScanData[i].scan.remocon;
			min = i;
		}
		m_iPtScanLink[index] = min;
		fMark[min] = true;
	}
	m_iPtScanCount -= levelLow;
	for(index = 0; index < m_iPtScanCount; ++index){
		int remocon = m_iPtScanLink[index];
		StringCchPrintf(sz,NUMELMS(sz),L"%2d:%2.2sch %s",
			m_sPtScanData[remocon].scan.remocon,
			m_sPtScanData[remocon].chName,
			m_sPtScanData[remocon].scan.nitCannelName);
		SendMessage(hItem,CB_ADDSTRING,index,(LPARAM )sz);
	}
	if(m_iPtScanCount){
		SendMessage(hItem,CB_SETCURSEL,m_iTunerRemocon = 0,0);
		int remocon = m_iPtScanLink[m_iTunerRemocon];
		m_iChannel = m_sPtScanData[remocon].channel;
	}

}
