//------------------------------------------------------------------------------
// File: ptDriver.cpp
//
// Date: 2011.05.11
//
// Written by Mr. Wakai.
//------------------------------------------------------------------------------

#include "ptDriver.h"

/**********************************************
 *
 *  CPtCapture Class
 *
 **********************************************/
CPtCapture::CPtCapture()
	: m_hPtDevice(INVALID_HANDLE_VALUE)
	, m_hFile(INVALID_HANDLE_VALUE)
	, m_fRun(false)
	, m_iChannel(0)
{

	ZeroMemory(m_wcCannelNo   ,sizeof(m_wcCannelNo));
	ZeroMemory(m_wcNetWorkName,sizeof(m_wcNetWorkName));
	ZeroMemory(m_wcCannelName ,sizeof(m_wcCannelName));
	ZeroMemory(m_wcRecFileName,sizeof(m_wcRecFileName));
}


CPtCapture::~CPtCapture()
{

	m_fRun = false;
	if(m_hPtDevice != INVALID_HANDLE_VALUE){
		PtDrvClose(m_hPtDevice);
		m_hPtDevice = INVALID_HANDLE_VALUE;
	}
}


HRESULT CPtCapture::SetPtDevName(LPCOLESTR pszDevName)
{

	if(wcslen(pszDevName) >= NUMELMS(m_wcPtDevName))	return E_INVALIDARG;
	StringCchCopy(m_wcPtDevName,NUMELMS(m_wcPtDevName),pszDevName);

	if(m_hPtDevice != INVALID_HANDLE_VALUE){
		PtDrvClose(m_hPtDevice);
		m_hPtDevice = INVALID_HANDLE_VALUE;
	}
	if((m_hPtDevice = PtDrvOpen(m_wcPtDevName)) == INVALID_HANDLE_VALUE)	return E_INVALIDARG;
	return S_OK;
}

HRESULT CPtCapture::GetPtDevName(LPOLESTR pszDevName)
{

	return StringCchCopy(pszDevName,NUMELMS(m_wcPtDevName),m_wcPtDevName);
}

void CPtCapture::ReceiveCapture()
{
	UINT	tsSize;
	DWORD	nBytesWrite;

	while(m_fRun){
		if(PtDrvGetStreamSize(m_hPtDevice,&tsSize) != NOERROR)	return;
		if(tsSize == 0)	return;
		if(tsSize >= sizeof(m_bPtTsBuf))	tsSize = sizeof(m_bPtTsBuf);
		PtDrvGetStreamData(m_hPtDevice,m_bPtTsBuf,tsSize,&tsSize);
		WriteFile(m_hFile,m_bPtTsBuf,tsSize,&nBytesWrite,NULL);
	}
}

HRESULT CPtCapture::GetPtChMax(int *pChannelMax)
{

	if(m_hPtDevice == INVALID_HANDLE_VALUE)	return E_INVALIDARG;
	PtDrvGetChannelMax(m_hPtDevice,pChannelMax);
	return NOERROR;
}

HRESULT CPtCapture::GetPtChInfo(int channel,int *pFrequency,wchar_t const **ppChName)
{
	char const	*pChName;

	if(m_hPtDevice == INVALID_HANDLE_VALUE)	return E_INVALIDARG;
	PtDrvGetChannelInfo(m_hPtDevice,channel,pFrequency,&pChName);
	MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,pChName,-1,m_wcCannelNo,NUMELMS(m_wcCannelNo));
	*ppChName = m_wcCannelNo;
	return NOERROR;
}

HRESULT CPtCapture::ExePtTunerScan(int channel,ptScanData *pScan)
{
	bool	fSleep;
	TMCC_STATUS	tmcc;
	DWORD		strTime;
	UINT		tsSize;
 	HRESULT		hr;

	if(m_hPtDevice == INVALID_HANDLE_VALUE)	return E_INVALIDARG;

	hr = NOERROR;
	PtDrvGetTunerSleep(m_hPtDevice,&fSleep);
	if(fSleep)	PtDrvSetTunerSleep(m_hPtDevice,false);
	else	PtDrvSetStreamEnable(m_hPtDevice,false);

	if(PtDrvSetFrequency(m_hPtDevice,(UINT )channel) != NOERROR)	hr = E_INVALIDARG;
	else{
		SleepEx(10,FALSE);
		if(PtDrvGetTmcc(m_hPtDevice,&tmcc))	hr = E_INVALIDARG;
		else{
			UINT	level,currentAgc,maxAgc;
			PtDrvGetCnAgc(m_hPtDevice,&level,&currentAgc,&maxAgc);
			if(level < 2000)	hr = E_INVALIDARG;
			else{
				PtDrvSetStreamEnable(m_hPtDevice,true);
				strTime = timeGetTime();
				m_iNitOffset = 0;
				for(hr = E_INVALIDARG; (timeGetTime() - strTime) < 1500; SleepEx(10,FALSE)){
					do{
						PtDrvGetStreamData(m_hPtDevice,m_bPtTsBuf,sizeof(m_bPtTsBuf),&tsSize);
						if(tsSize == 0)	continue;
						if(!RcvNitSection(m_bPtTsBuf,tsSize))	continue;
						hr = NOERROR;
						break;
					}while(tsSize);
					if(hr != NOERROR)	continue;
					pScan->remocon = m_iNitRemocon;
					pScan->level = level;
					wcscpy_s(pScan->netWorkName,NUMELMS(pScan->netWorkName),m_wcNetWorkName);
					wcscpy_s(pScan->nitCannelName,NUMELMS(pScan->nitCannelName),m_wcCannelName);
					break;
				}
			}
		}
	}
	PtDrvSetTunerSleep(m_hPtDevice,true);

	return hr;
}

bool CPtCapture::RcvNitSection(const BYTE *pPkt,size_t pktSize)
{
	int		len;
	int		size;
	const BYTE	*pSec;

	for(int n = pktSize / PACKET_SIZE; n--; pPkt += PACKET_SIZE){
		if(pPkt[1] & TRANS_INDICATOR)	continue;
		if(GetPktPid(pPkt) != PID_NIT)	continue;
		if(pPkt[1] & PAYLOAD_START){
			m_iNitSize = GetSectionSize(pPkt) + 3;
			pSec = GetPktData(pPkt,size);
			if(*pSec != TID_NIT)	return false;
			CopyMemory(m_bNitSec,pSec,size);
			m_iNitOffset = size;
		}else{
			if(m_iNitOffset == 0)	continue;
			if(m_iNitPktCnt != GET_PKT_CNT(pPkt[3])){
				m_iNitOffset = 0;
				continue;
			}
			len = m_iNitSize - m_iNitOffset;
			pSec = GetPktData(pPkt,size);
			if(size > len)	size = len;
			CopyMemory(m_bNitSec + m_iNitOffset,pSec,size);
			m_iNitOffset += size;
		}
		m_iNitPktCnt = GET_PKT_CNT(pPkt[3] + 1);
		if(m_iNitSize > m_iNitOffset)	continue;
		m_iNitOffset = 0;
		if(!DrvChkSection(m_bNitSec,m_iNitSize))	continue;
		if(RcvNitAnalyze(m_bNitSec,m_iNitSize))	return true;
	}
	return false;
}

bool CPtCapture::RcvNitAnalyze(const BYTE *pNit,size_t nitSize)
{
	int		n,infoLen;
	int		len;
	int		size;
	const BYTE	*pSrc;
	BYTE	tmp[256];

	if(pNit[0] != TID_NIT)	return false;
	len = infoLen = GetTsLen(pNit + 8);
	m_wcNetWorkName[0] = L'\0';
	for(pSrc = pNit + 10; len > 0; pSrc += (2 +  n)){
		n = pSrc[1];
		if(*pSrc != TAG_NT_TG)	continue;
		memcpy(tmp,pSrc + 2,n); tmp[n] = 0x00;
		DrvCnvEuni2uni(tmp,m_wcNetWorkName,NUMELMS(m_wcNetWorkName));
		break;
	}
	if(len < 0)	return false;
	pSrc = pNit + (10 + infoLen);
	len  = GetTsLen(pSrc + 6);
	for(pSrc += 2 + 6; len > 0; pSrc += n){
		n = pSrc[1] + 2;
		len -= n;
		if(*pSrc != 0xcd)	continue;
		m_iNitRemocon = pSrc[2];
		size = pSrc[3] >> 2;
		memcpy(tmp,pSrc + 4,size); tmp[size] = 0x00;
		DrvCnvEuni2uni(tmp,m_wcCannelName,NUMELMS(m_wcCannelName));
		return true;
	}
	return false;
}

HRESULT CPtCapture::SetPtTunerTuning(int channel)
{
	bool	fSleep;
	TMCC_STATUS	tmcc;
 	HRESULT		hr;

	if(m_hPtDevice == INVALID_HANDLE_VALUE)	return E_INVALIDARG;

	hr = NOERROR;
	PtDrvGetTunerSleep(m_hPtDevice,&fSleep);
	if(fSleep)	PtDrvSetTunerSleep(m_hPtDevice,false);
	else	PtDrvSetStreamEnable(m_hPtDevice,false);

	if(PtDrvSetFrequency(m_hPtDevice,(UINT )channel) != NOERROR)	hr = E_INVALIDARG;
	else{
		SleepEx(10,FALSE);
		if(PtDrvGetTmcc(m_hPtDevice,&tmcc))	hr = E_INVALIDARG;
		else{
			UINT	level,currentAgc,maxAgc;
			PtDrvGetCnAgc(m_hPtDevice,&level,&currentAgc,&maxAgc);
			PtDrvSetStreamEnable(m_hPtDevice,true);
		}
	}
	return hr;
}

HRESULT CPtCapture::GetPtFilterStatus(BOOL *pReady)
{

	if(pReady == NULL)	return E_INVALIDARG;
	*pReady = (m_hPtDevice != INVALID_HANDLE_VALUE);
	return S_OK;
}

HRESULT CPtCapture::ExeStartRec(int channel,LPCOLESTR pszFileName)
{

	if(m_hPtDevice == INVALID_HANDLE_VALUE)	return E_INVALIDARG;
	if(wcslen(pszFileName) > NUMELMS(m_wcRecFileName))	return E_INVALIDARG;
	StringCchCopy(m_wcRecFileName,NUMELMS(m_wcRecFileName),pszFileName);
	if(m_fRun){
		if(m_hFile != INVALID_HANDLE_VALUE)	CloseHandle(m_hFile);
		m_fRun = false;
	}
	m_hFile = CreateFile(m_wcRecFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(m_hFile == INVALID_HANDLE_VALUE)	return E_INVALIDARG;

	m_fRun = true;
	m_iChannel = channel;

	SetPtTunerTuning(m_iChannel);
	return S_OK;
}

HRESULT CPtCapture::ExeStopRec()
{

	CloseHandle(m_hFile);
	PtDrvSetTunerSleep(m_hPtDevice,true);
	PtDrvSetStreamEnable(m_hPtDevice,false);
	m_fRun = false;
	return S_OK;
}

/**********************************************
 *
 *  CPtDriver Class
 *
 **********************************************/
CPtDriver::CPtDriver()
{

	m_pPtCapture = new CPtCapture;
	if(m_pPtCapture == NULL)	goto err_exit;

	m_fCaptureExit  = false;
	m_hCaptureEvent = CreateEvent(NULL,FALSE,FALSE,NULL);
	if(m_hCaptureEvent ==  NULL)	goto err_exit;
	m_hCaptureThread = CreateThread(NULL,0,CaptureThread,this,0,NULL);
	if(m_hCaptureThread == NULL)	goto err_exit;
	return;

err_exit:
	CPtExit();
}

void CPtDriver::CPtExit()
{

	SAFE_DELETE(m_pPtCapture);
	if(m_hCaptureEvent != NULL){ CloseHandle(m_hCaptureEvent); m_hCaptureEvent = NULL;}
}

CPtDriver::~CPtDriver()
{   

	if(m_hCaptureEvent != NULL){
		m_fCaptureExit = TRUE;
		SetEvent(m_hCaptureEvent);
		if(WaitForSingleObjectEx(m_hCaptureThread,TIMER_100ms,FALSE) != WAIT_OBJECT_0){
			TerminateThread(m_hCaptureThread,0);
		}
		m_hCaptureThread = NULL;
	}
	CPtExit();
}


DWORD WINAPI CPtDriver::CaptureThread(LPVOID pParam)
{
	CPtDriver	*pThis;

	pThis = (CPtDriver *)pParam;
	for( ; ; ){
		WaitForSingleObjectEx(pThis->m_hCaptureEvent,TIMER_3ms,FALSE);
		if(pThis->m_fCaptureExit == TRUE)	break;
		if(pThis->m_pPtCapture->IsRun() == false)	continue;
		pThis->m_pPtCapture->ReceiveCapture();
	};
	ExitThread(0);
	return 0;
}


HRESULT CPtDriver::set_devName(LPCOLESTR pszDevName)
{

	if(pszDevName == NULL)	return E_INVALIDARG;
	return m_pPtCapture->SetPtDevName(pszDevName);
}


HRESULT CPtDriver::get_devName(LPOLESTR pszDevName)
{

	if(pszDevName == NULL)	return E_INVALIDARG;
	return m_pPtCapture->GetPtDevName(pszDevName);
}


HRESULT CPtDriver::get_chMax(int *pChannelMax)
{

	return m_pPtCapture->GetPtChMax(pChannelMax);
}

HRESULT CPtDriver::get_chInfo(int channel,int *pFrequency,wchar_t const **ppChName)
{

	return m_pPtCapture->GetPtChInfo(channel,pFrequency,ppChName);
}

HRESULT CPtDriver::get_status(BOOL *pReady)
{

	return m_pPtCapture->GetPtFilterStatus(pReady);
}


HRESULT CPtDriver::exe_startScan(int channel,ptScanData *pScan)
{

	return m_pPtCapture->ExePtTunerScan(channel,pScan);
}


HRESULT CPtDriver::exe_startRec(int channel,LPCOLESTR pszFileName)
{

	return m_pPtCapture->ExeStartRec(channel,pszFileName);
}


HRESULT CPtDriver::exe_stopRec()
{

	return m_pPtCapture->ExeStopRec();
}


