//------------------------------------------------------------------------------
// File: CPtDriver.h
//
// Date: 2011.05.11
//
// Written by Mr. Wakai.
//------------------------------------------------------------------------------

#pragma once

#include <tchar.h>
#include <windows.h>
#include <assert.h>
#include <strsafe.h>

#include "PtDrvIfLib.h"

#ifndef NUMELMS
#define NUMELMS(aa)	(sizeof(aa)/sizeof((aa)[0]))
#endif

#ifndef	STR_MAX_LENGTH
#define	STR_MAX_LENGTH	(256)
#endif

#ifndef SAFE_RELEASE
#define SAFE_RELEASE(pObject) if(pObject){ pObject->Release(); pObject = NULL;}
#endif

#ifndef SAFE_DELETE
#define SAFE_DELETE(pObject) if(pObject){ delete pObject; pObject = NULL;}
#endif

#ifndef ASSERT
#define ASSERT(x) assert(x)
#endif


#define	PACKET_SIZE			(188)
#define	SECTION_MAX_SIZE	(0x1000)
#define	PT_BUF_SIZE			(PACKET_SIZE * 8192)

#define	TRANS_INDICATOR		(0x80)
#define	PAYLOAD_START		(0x40)
#define	TRANS_PRIORITY		(0x20)

#define	ADPT_PLYLD			(0x10)
#define	ADPT_ADPT			(0x20)

#define	PID_MASK			(0x1fff)
#define	PKT_CNT_MASK		(0x0f)
#define	SEC_LEN_MASK		(0x0fff)
#define	LENGTH_MASK			(0x0fff)

#define	SECTION_MAX_SIZE	(0x1000)

#define	TIMER_3ms		(3)
#define	TIMER_10ms		(10)
#define	TIMER_100ms		(100)

#define	PID_PAT			(0x0000)
#define	PID_NIT			(0x0010)
#define	PID_NULL		(0x1fff)

#define	TID_PAT			(0x00)
#define	TID_PMT			(0x02)
#define	TID_NIT			(0x40)

#define	PID_MASK				(0x1fff)

#define	GET_PKT_CNT(d)	(d & PKT_CNT_MASK)
#define	CRC_SIZE		(4)

#define	TAG_NT_TG		(0x40)

typedef	struct{
	int		remocon;
	UINT	level;
	wchar_t	netWorkName[32];
	wchar_t	nitCannelName[32];
}ptScanData;

/**********************************************
 *
 *  CPtCapture Class declarations
 *
 **********************************************/
class CPtCapture{


protected:
	int		m_iChannel;

public:
	CPtCapture();
	~CPtCapture();

	HRESULT StartPlay(void);

	HRESULT SetPtDevName(LPCOLESTR pszDevName);
	HRESULT GetPtDevName(LPOLESTR pszDevName);
	HRESULT GetPtChMax(int *pChannelMax);
	HRESULT GetPtChInfo(int channel,int *pFrequency,wchar_t const **ppChName);
	HRESULT ExePtTunerScan(int channel,ptScanData *pScan);
	HRESULT SetPtTunerTuning(int channel);
	HRESULT GetPtFilterStatus(BOOL *pReady);
	HRESULT ExeStartRec(int channel,LPCOLESTR pszDevName);
	HRESULT ExeStopRec();

	bool IsRun(){ return m_fRun; }
	void ReceiveCapture();

private:
	HANDLE	m_hPtDevice;
	HANDLE	m_hFile;

	BYTE	m_bPtTsBuf[PT_BUF_SIZE];

	wchar_t	m_wcPtDevName[MAX_PATH];
	wchar_t	m_wcRecFileName[MAX_PATH];

	bool	m_fRun;

	int		m_iPmtPid;
	int		m_iPmtNum;
	int		m_iPmtLength,m_iPmtOffset;
	int		m_iPmtPktCnt;
	int		m_iNitOffset;
	int		m_iNitSize;
	int		m_iNitPktCnt;
	int		m_iNitRemocon;
	BYTE	m_bPmtSec[SECTION_MAX_SIZE+3];
	BYTE	m_bNitSec[SECTION_MAX_SIZE+3];

	wchar_t	m_wcCannelNo[32];
	wchar_t	m_wcNetWorkName[32];
	wchar_t	m_wcCannelName[32];

	bool RcvNitSection(const BYTE *pBuf,size_t size);
	bool RcvNitAnalyze(const BYTE *pNit,size_t size);

	int GetPktPid(const BYTE *pPkt){ return (((pPkt[1] << 8) | pPkt[2]) & PID_MASK);};
	int GetPktLen(const BYTE *pLen){ return (((pLen[1] << 8) | pLen[2]) & SEC_LEN_MASK);};
	int GetTsWord(const BYTE *pTs){ return ((pTs[0] << 8) | pTs[1]);};
	int GetTsPid(const BYTE *pTs){ return (GetTsWord(pTs) & PID_MASK);};
	int GetTsLen(const BYTE *pTs){ return (GetTsWord(pTs) & LENGTH_MASK);};
	int GetSectionSize(const BYTE *pPkt){
		int offset = pPkt[4];
		if(offset > (PACKET_SIZE - 5 - 7))	return 0;
		pPkt = pPkt + (5 + offset);
		int length = ((pPkt[1] << 8) | pPkt[2]) & SEC_LEN_MASK;
		return length | ((length == 0) << 12);
	};
	const BYTE *GetPktData(const BYTE *pPkt,int &size){
		int offset = 0;
		if(pPkt[1] & PAYLOAD_START)	offset = 1;
		if(pPkt[3] & ADPT_ADPT)	offset += (pPkt[4] + 1);
		size = (PACKET_SIZE - 4) - offset;
		return (pPkt + (4 + offset));
	}

};

/**********************************************
 *
 *  CPtDriver Class declarations
 *
 **********************************************/
class CPtDriver 
{
	CPtCapture	*m_pPtCapture;

protected:
	HANDLE	m_hCaptureEvent;
	HANDLE	m_hCaptureThread;
	BOOL	m_fCaptureExit;

public:
	CPtDriver();
	~CPtDriver();

	HRESULT set_devName(LPCOLESTR pszDevName);
	HRESULT get_devName(LPOLESTR pszDevName);
	HRESULT get_chMax(int *pChannelMax);
	HRESULT get_chInfo(int channel,int *pFrequency,wchar_t const **ppChName);
	HRESULT set_chTuning(int channel);
	HRESULT get_status(BOOL *pReady);

	HRESULT exe_startScan(int channel,ptScanData *pScan);
	HRESULT exe_startRec(int channel,LPCOLESTR pszFileName);
	HRESULT exe_stopRec();

private:
	void CPtExit();
	static	DWORD WINAPI CaptureThread(LPVOID pParam);

};

